@@ -9,7 +9,7 @@ Michael Hucka <mhucka@caltech.edu> -- Caltech Library
 Copyright
 ---------
 
-Copyright (c) 2018-2020 by the California Institute of Technology.  This code
+Copyright (c) 2018-2022 by the California Institute of Technology.  This code
 is open-source software released under a 3-clause BSD license.  Please see the
 file "LICENSE" for more information.
 '''
@@ -105,12 +105,14 @@ def on_localhost(url):
         try:
             addrinfo = socket.getaddrinfo(host, None, family, socket.SOCK_STREAM)
         except socket.gaierror as ex:
-            log(f'socket.getaddrinfo exception: {antiformat(ex)}')
-            return False
-        for family, _, _, _, sockaddr in addrinfo:
-            address_part = sockaddr[0]
-            if ip_address(address_part).is_loopback:
-                return True
+            break
+        else:
+            for _, _, _, _, sockaddr in addrinfo:
+                address_part = sockaddr[0]
+                if ip_address(address_part).is_loopback:
+                    log(f'address seems to be on localhost: {url}')
+                    return True
+    log(f'address not on localhost: {url}')
     return False
 
 
@@ -157,14 +159,18 @@ def timed_request(method, url, client = None, **kwargs):
             else:
                 failures += 1
         except KeyboardInterrupt as ex:
-            if __debug__: log(addurl(f'network {method} interrupted by {str(ex)}'))
+            if __debug__: log(addurl(f'network {method} interrupted by {antiformat(ex)}'))
             raise
+        except TypeError as ex:
+            # Bad arguments to the call, like passing data to a 'get'.
+            if __debug__: log(addurl(f'exception {antiformat(ex)}'))
+            raise ArgumentError(f'Bad or invalid arguments in network call')
         except (httpx.CookieConflict, httpx.StreamError, httpx.TooManyRedirects,
                 httpx.DecodingError, httpx.ProtocolError, httpx.ProxyError,
                 httpx.ConnectError) as ex:
             # Probably indicates a deeper issue.  Don't do our lengthy retry
             # sequence, but try one more time, in case it's transient.
-            if __debug__: log(addurl(f'exception {str(ex)}'))
+            if __debug__: log(addurl(f'exception {antiformat(ex)}'))
             if failures > 0:
                 raise
             failures += 1
@@ -172,7 +178,7 @@ def timed_request(method, url, client = None, **kwargs):
         except Exception as ex:
             # Problem might be transient.  Don't quit right away.
             failures += 1
-            if __debug__: log(addurl(f'exception (failure #{failures}): {str(ex)}'))
+            if __debug__: log(addurl(f'exception (failure #{failures}): {antiformat(ex)}'))
             # Record the first error we get, not the subsequent ones, because
             # in the case of network outages, the subsequent ones will be
             # about being unable to reconnect and not the original problem.
@@ -225,7 +231,7 @@ def net(method, url, client = None, handle_rate = True,
     the URL.  It is up to the caller to implement the polling schedule and
     call this function (with polling = True) as needed.
 
-    This method always passes the argument allow_redirects = True to the
+    This method always passes the argument follow_redirects = True to the
     underlying Python HTTPX library network calls.
     '''
     known_methods = ['get', 'post', 'head', 'options', 'put', 'delete', 'patch']
@@ -238,19 +244,24 @@ def net(method, url, client = None, handle_rate = True,
 
     resp = None
     try:
-        resp = timed_request(method, url, client, allow_redirects = True, **kwargs)
+        resp = timed_request(method, url, client, follow_redirects = True, **kwargs)
     except (httpx.NetworkError, httpx.ProtocolError) as ex:
         # timed_request() will have retried, so if we get here, time to bail.
-        if __debug__: log(addurl(f'got network exception: {str(ex)}'))
-        if on_localhost(url) or network_available():
-            if __debug__: log(addurl('failed > 1 times -- returning ServiceFailure'))
-            return (resp, ServiceFailure(addurl('Network or server error {str(ex)}')))
+        if __debug__: log(addurl(f'got network exception: {antiformat(ex)}'))
+        is_on_localhost = on_localhost(url)
+        msg = antiformat(ex)
+        if isinstance(ex, httpx.ConnectError) and is_on_localhost:
+            if __debug__: log(addurl('returning ServiceFailure'))
+            return (resp, ServiceFailure(addurl(f'Access failure ({msg})')))
+        elif is_on_localhost or network_available():
+            if __debug__: log(addurl('returning ServiceFailure'))
+            return (resp, ServiceFailure(addurl(f'Server error ({msg})')))
         else:
             if __debug__: log(addurl('returning NetworkFailure'))
-            return (resp, NetworkFailure(addurl('Network connectivity failure')))
+            return (resp, NetworkFailure(addurl('Network failure ({msg})')))
     except Exception as ex:
         # Not a network or protocol error, and not a normal server response.
-        if __debug__: log(addurl(f'returning exception: {str(ex)}'))
+        if __debug__: log(addurl(f'returning exception: {antiformat(ex)}'))
         return (resp, ex)
 
     # Interpret the response.  Note that the requests library handles code 301
@@ -267,7 +278,7 @@ def net(method, url, client = None, handle_rate = True,
     elif code in [405, 406, 409, 411, 412, 413, 414, 417, 428, 431, 505, 510]:
         error = InternalError(addurl(f'Server returned code {code} ({reason})'))
     elif code in [415, 416]:
-        error = ServiceFailure(addurl('Server rejected the request ({reason})'))
+        error = ServiceFailure(addurl(f'Server rejected the request ({reason})'))
     elif code == 429:
         if handle_rate and recursing < _MAX_RECURSIVE_CALLS:
             pause = 5 * (recursing + 1)   # +1 b/c we start with recursing = 0.
@@ -295,7 +306,7 @@ def download_file(url, local_destination):
     try:
         download(url, local_destination)
     except Exception as ex:
-        if __debug__: log(f'download exception: {str(ex)}')
+        if __debug__: log(f'download exception: {antiformat(ex)}')
         return False
     else:
         return True
@@ -306,7 +317,7 @@ def download(url, local_destination, recursing = 0):
 
     timeout = httpx.Timeout(15, connect = 15, read = 15, write = 15)
     with httpx.stream('get', url, verify = False, timeout = timeout,
-                      allow_redirects = True) as resp:
+                      follow_redirects = True) as resp:
         code = resp.status_code
         if code == 202:
             # Code 202 = Accepted, "received but not yet acted upon."
@@ -341,4 +352,4 @@ def download(url, local_destination, recursing = 0):
         elif code in [500, 501, 502, 506, 507, 508]:
             raise ServiceFailure(addurl(f'Internal server error (HTTP code {code})'))
         else:
-            raise NetworkFailure('Unable to resolve {}'.format(url))
+            raise NetworkFailure(f'Unable to resolve {url}')
